# Generated by Django 3.2.13 on 2022-06-22 15:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0025_schoolteacherinvitation'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='teacher',
            name='pending_join_request',
        ),
        migrations.AlterField(
            model_name='teacher',
            name='blocked_time',
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.2.13 on 2022-05-03 11:01

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0021_school_is_active'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='school',
            name='latitude',
        ),
        migrations.RemoveField(
            model_name='school',
            name='longitude',
        ),
        migrations.RemoveField(
            model_name='school',
            name='town',
        ),
        migrations.RemoveField(
            model_name='userprofile',
            name='can_view_aggregated_data',
        ),
    ]



import pytest
from django.db.models.query import QuerySet


@pytest.mark.django_db
def test_teacher_title_removed(migrator):
    old_state = migrator.apply_initial_migration(("common", "0009_add_blocked_time_to_teacher_and_student"))
    Teacher = old_state.apps.get_model("common", "Teacher")
    assert hasattr(Teacher, "title")

    new_state = migrator.apply_tested_migration(("common", "0010_remove_teacher_title"))
    Teacher = new_state.apps.get_model("common", "Teacher")
    assert not hasattr(Teacher, "title")



import pytest
from django.db.models.query import QuerySet


@pytest.mark.django_db
def test_characters_added(migrator):
    migrator.apply_initial_migration(("common", "0002_emailverification"))
    new_state = migrator.apply_tested_migration(("common", "0004_add_aimmocharacters"))

    model_names = [model._meta.db_table for model in new_state.apps.get_models()]

    assert "common_aimmocharacter" in model_names

    AimmoCharacter = new_state.apps.get_model("common", "aimmocharacter")
    all_characters: QuerySet = AimmoCharacter.objects.all()

    assert all_characters.count() == 3


@pytest.mark.django_db
def test_image_paths_updated(migrator):
    migrator.apply_initial_migration(("common", "0005_add_worksheets"))
    new_state = migrator.apply_tested_migration(("common", "0006_update_aimmo_character_image_path"))

    AimmoCharacter = new_state.apps.get_model("common", "aimmocharacter")
    all_characters: QuerySet = AimmoCharacter.objects.all()

    for character in all_characters:
        assert character.image_path.startswith("images/aimmo_characters/")



# Generated by Django 3.2.13 on 2022-06-07 17:20

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0023_userprofile_aimmo_badges"),
    ]

    operations = [
        migrations.AddField(
            model_name="teacher",
            name="invited_by",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="invited_teachers",
                to="common.teacher",
            ),
        ),
    ]



# Generated by Django 2.2.24 on 2021-08-18 15:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0009_add_blocked_time_to_teacher_and_student"),
        ("portal", "0062_verify_portaladmin"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="teacher",
            name="title",
        ),
    ]



import random
import string
import hashlib
from builtins import range, str
from uuid import uuid4

from common.models import Class, Student
from django.contrib.auth.models import User


def get_random_username():
    while True:
        random_username = uuid4().hex[:30]  # generate a random username
        if not User.objects.filter(username=random_username).exists():
            return random_username


def generate_new_student_name(orig_name):
    if not Student.objects.filter(new_user__username=orig_name).exists():
        return orig_name

    i = 1
    while True:
        new_name = orig_name + str(i)
        if not Student.objects.filter(new_user__username=new_name).exists():
            return new_name
        i += 1


def generate_access_code():
    while True:
        access_code = "".join(random.choice(string.ascii_uppercase) for _ in range(5))

        if not Class.objects.filter(access_code=access_code).exists():
            return access_code


def generate_password(length):
    return "".join(random.choice(string.ascii_lowercase) for _ in range(length))


def generate_login_id():
    """Returns the uuid string and its hashed.
    The string is used for URL, and the hashed is stored in the DB."""
    login_id = uuid4().hex
    hashed_login_id = get_hashed_login_id(login_id)
    return login_id, hashed_login_id


def get_hashed_login_id(login_id):
    """Returns the hash of a given string used for login url"""
    return hashlib.sha256(login_id.encode()).hexdigest()



# Generated by Django 3.2.15 on 2022-10-12 12:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('common', '0028_coding_club_downloads'),
    ]

    operations = [
        migrations.CreateModel(
            name='DynamicElement',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=64, unique=True, editable=False)),
                ('active', models.BooleanField(default=False)),
                ('text', models.TextField(blank=True, null=True)),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-09-10 08:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
from django.db.migrations.operations.special import SeparateDatabaseAndState
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("common", "0001_initial"),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name="EmailVerification",
                    fields=[
                        (
                            "id",
                            models.AutoField(
                                auto_created=True,
                                primary_key=True,
                                serialize=False,
                                verbose_name="ID",
                            ),
                        ),
                        ("token", models.CharField(max_length=30)),
                        (
                            "email",
                            models.CharField(
                                blank=True, default=None, max_length=200, null=True
                            ),
                        ),
                        ("expiry", models.DateTimeField()),
                        ("verified", models.BooleanField(default=False)),
                        (
                            "user",
                            models.ForeignKey(
                                blank=True,
                                null=True,
                                on_delete=django.db.models.deletion.CASCADE,
                                related_name="email_verifications",
                                to=settings.AUTH_USER_MODEL,
                            ),
                        ),
                    ],
                )
            ],
            # Table already exists. See portal/migrations/0058_move_to_common_models.py
            database_operations=[],
        ),
    ]



# Generated by Django 2.2.24 on 2021-10-12 16:22

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("common", "0011_student_login_id"),
    ]

    operations = [
        migrations.CreateModel(
            name="UserSession",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("login_time", models.DateTimeField(default=django.utils.timezone.now)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.24 on 2020-09-23 10:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("common", "0002_emailverification")]

    operations = [
        migrations.CreateModel(
            name="AimmoCharacter",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255)),
                ("description", models.TextField()),
                ("image_path", models.CharField(max_length=255)),
                ("sort_order", models.IntegerField()),
            ],
        )
    ]



from common.helpers.data_migration_loader import load_data_from_file
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("common", "0017_copy_email_to_username")]

    operations = [
        migrations.AddField(
            "AimmoCharacter",
            "alt",
            models.CharField(max_length=255, null=True),
        ),
    ]



from pathlib import Path
from typing import Callable

from django.core.management import call_command
from django.core.serializers import base, python


def load_data_from_file(file_name) -> Callable:
    """Returns a migration function that loads a json file produced by `manage.py dumpdata` into the database.
    For use with migrations.RunPython

    Args:
        file_name (str): The name of the file containing the data you want to load. Include `.json` at the end. The file must be in the fixtures directory.
    """
    absolute_file_path = Path(__file__).resolve().parent.parent / "fixtures" / file_name

    def _load_fixture(apps, schema_editor):
        # Save the default _get_model() function
        default_get_model = python._get_model

        # Define new _get_model() function here, which utilizes the apps argument to
        # get the historical version of a model. This piece of code is directly taken
        # from django.core.serializers.python._get_model, unchanged. However, here it
        # has a different context, specifically, the apps variable.
        def _get_model(model_identifier):
            try:
                return apps.get_model(model_identifier)
            except (LookupError, TypeError):
                raise base.DeserializationError(
                    "Invalid model identifier: '%s'" % model_identifier
                )

        # Replace the _get_model() function on the module, so loaddata can utilize it.
        python._get_model = _get_model

        try:
            # Call loaddata command
            call_command("loaddata", absolute_file_path, app_label="common")
        finally:
            # Restore default _get_model() function
            python._get_model = default_get_model

    return _load_fixture



# Generated by Django 2.0 on 2020-10-28 17:36

from django.db import migrations


def add_maintenance_banner(apps, schema_editor):
    """
    This creates the maintenance banner dynamic element which allows us to edit and turn on and off the maintenance
    banner dynamically from the admin panel.
    """
    DynamicElement = apps.get_model("common", "DynamicElement")
    DynamicElement.objects.create(name="Maintenance banner")


def remove_maintenance_banner(apps, schema_editor):
    DynamicElement = apps.get_model("common", "DynamicElement")
    maintenance_banner = DynamicElement.objects.get(name="Maintenance banner")
    maintenance_banner.delete()


class Migration(migrations.Migration):

    dependencies = [("common", "0029_dynamicelement")]

    operations = [
        migrations.RunPython(add_maintenance_banner, remove_maintenance_banner)
    ]



# Generated by Django 2.2.20 on 2021-05-05 11:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0008_unlock_worksheet_3"),
        ("portal", "0058_move_to_common_models"),
    ]

    operations = [
        migrations.AddField(
            model_name="student",
            name="blocked_time",
            field=models.DateTimeField(null=True),
        ),
        migrations.AddField(
            model_name="teacher",
            name="blocked_time",
            field=models.DateTimeField(null=True),
        ),
    ]



# Generated by Django 2.2.24 on 2021-12-06 15:03

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0014_login_type"),
    ]

    operations = [
        migrations.CreateModel(
            name="DailyActivity",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("date", models.DateField(default=django.utils.timezone.now)),
                ("csv_click_count", models.PositiveIntegerField(default=0)),
                ("login_cards_click_count", models.PositiveIntegerField(default=0)),
            ],
        ),
    ]



from common.helpers.data_migration_loader import load_data_from_file
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("common", "0018_update_aimmo_character_image_path"),
    ]

    operations = [
        migrations.RunPython(
            load_data_from_file("aimmo_characters3.json"),
            reverse_code=migrations.RunPython.noop,
        )
    ]



# -*- coding: utf-8 -*-
import re
from setuptools import find_packages, setup

with open("../portal/__init__.py", "r") as fd:
    version = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', fd.read(), re.MULTILINE
    ).group(1)

setup(
    name="cfl-common",
    packages=find_packages(),
    version=version,
    include_package_data=True,
    install_requires=[
        "django==3.2.16",
        "djangorestframework==3.13.1",
        "django-two-factor-auth==1.13.2",
        "django-countries==7.3.1",
    ],
    tests_require=[],
    test_suite="tests",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
)



import pytest


@pytest.mark.django_db
def test_blocked_time_added(migrator):
    migrator.apply_initial_migration(("common", "0008_unlock_worksheet_3"))
    new_state = migrator.apply_tested_migration(("common", "0009_add_blocked_time_to_teacher_and_student"))

    teacher_model = new_state.apps.get_model("common", "Teacher")

    assert teacher_model._meta.get_field("blocked_time").get_internal_type() == "DateTimeField"

    student_model = new_state.apps.get_model("common", "Student")

    assert student_model._meta.get_field("blocked_time").get_internal_type() == "DateTimeField"



import datetime
import json
from enum import Enum, auto
from uuid import uuid4

from common import app_settings
from common.app_settings import domain
from common.email_messages import (
    emailChangeNotificationEmail,
    emailChangeVerificationEmail,
    emailChangeDuplicateNotificationEmail,
    emailVerificationNeededEmail,
    parentsEmailVerificationNeededEmail,
)
from common.models import EmailVerification, Teacher, Student
from django.contrib.auth.models import User
from django.core.mail import EmailMultiAlternatives
from django.http import HttpResponse
from django.template import loader
from django.utils import timezone
from requests import post, get, put, delete
from requests.exceptions import RequestException


NOTIFICATION_EMAIL = "Code For Life Notification <" + app_settings.EMAIL_ADDRESS + ">"
VERIFICATION_EMAIL = "Code For Life Verification <" + app_settings.EMAIL_ADDRESS + ">"
PASSWORD_RESET_EMAIL = "Code For Life Password Reset <" + app_settings.EMAIL_ADDRESS + ">"
INVITE_FROM = "Code For Life Invitation <" + app_settings.EMAIL_ADDRESS + ">"


class DotmailerUserType(Enum):
    TEACHER = auto()
    STUDENT = auto()
    NO_ACCOUNT = auto()


def send_email(
    sender,
    recipients,
    subject,
    text_content,
    title,
    html_content=None,
    plaintext_template="email.txt",
    html_template="email.html",
):

    # add in template for templates to message

    # setup templates
    plaintext = loader.get_template(plaintext_template)
    html = loader.get_template(html_template)
    plaintext_email_context = {"content": text_content}
    html_email_context = {"content": text_content, "title": title, "url_prefix": domain()}

    # render templates
    plaintext_body = plaintext.render(plaintext_email_context)
    html_body = html.render(html_email_context)

    # make message using templates
    message = EmailMultiAlternatives(subject, plaintext_body, sender, recipients)
    message.attach_alternative(html_body, "text/html")

    message.send()


def generate_token(user, email="", preverified=False):
    return EmailVerification.objects.create(
        user=user,
        email=email,
        token=uuid4().hex[:30],
        expiry=timezone.now() + datetime.timedelta(hours=1),
        verified=preverified,
    )


def _newsletter_ticked(data):
    return data["newsletter_ticked"]


def send_verification_email(request, user, new_email=None, age=None, data=None):
    """Send an email prompting the user to verify their email address."""

    if not new_email:  # verifying first email address
        user.email_verifications.all().delete()

        verification = generate_token(user)

        if age is not None and age < 13:
            message = parentsEmailVerificationNeededEmail(request, user, verification.token)
            send_email(VERIFICATION_EMAIL, [user.email], message["subject"], message["message"], message["subject"])
        else:
            if data is not None and _newsletter_ticked(data):
                add_to_dotmailer(user.first_name, user.last_name, user.email, DotmailerUserType.STUDENT)
            message = emailVerificationNeededEmail(request, verification.token)
            send_email(VERIFICATION_EMAIL, [user.email], message["subject"], message["message"], message["subject"])

    else:  # verifying change of email address.
        verification = generate_token(user, new_email)

        message = emailChangeVerificationEmail(request, verification.token)
        send_email(VERIFICATION_EMAIL, [user.email], message["subject"], message["message"], message["subject"])

        message = emailChangeNotificationEmail(request, new_email)
        send_email(VERIFICATION_EMAIL, [user.email], message["subject"], message["message"], message["subject"])


def is_verified(user):
    """Check that a user has verified their email address."""
    verifications = user.email_verifications.filter(verified=True)
    return len(verifications) != 0


def add_to_dotmailer(first_name: str, last_name: str, email: str, user_type: DotmailerUserType):
    try:
        create_contact(first_name, last_name, email)
        add_contact_to_address_book(first_name, last_name, email, user_type)
    except RequestException:
        return HttpResponse(status=404)


def create_contact(first_name, last_name, email):
    url = app_settings.DOTMAILER_CREATE_CONTACT_URL
    body = {
        "contact": {
            "email": email,
            "optInType": "VerifiedDouble",
            "emailType": "Html",
            "dataFields": [
                {"key": "FIRSTNAME", "value": first_name},
                {"key": "LASTNAME", "value": last_name},
                {"key": "FULLNAME", "value": f"{first_name} {last_name}"},
            ],
        },
        "consentFields": [{"fields": [{"key": "DATETIMECONSENTED", "value": datetime.datetime.now().__str__()}]}],
        "preferences": app_settings.DOTMAILER_DEFAULT_PREFERENCES,
    }

    post(url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))


def add_contact_to_address_book(first_name: str, last_name: str, email: str, user_type: DotmailerUserType):
    main_address_book_url = app_settings.DOTMAILER_MAIN_ADDRESS_BOOK_URL

    body = {
        "email": email,
        "optInType": "VerifiedDouble",
        "emailType": "Html",
        "dataFields": [
            {"key": "FIRSTNAME", "value": first_name},
            {"key": "LASTNAME", "value": last_name},
            {"key": "FULLNAME", "value": f"{first_name} {last_name}"},
        ],
    }

    post(main_address_book_url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))

    specific_address_book_url = app_settings.DOTMAILER_NO_ACCOUNT_ADDRESS_BOOK_URL

    if user_type == DotmailerUserType.TEACHER:
        specific_address_book_url = app_settings.DOTMAILER_TEACHER_ADDRESS_BOOK_URL
    elif user_type == DotmailerUserType.STUDENT:
        specific_address_book_url = app_settings.DOTMAILER_STUDENT_ADDRESS_BOOK_URL

    post(specific_address_book_url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))


def delete_contact(email: str):
    try:
        user = get_dotmailer_user_by_email(email)
        user_id = user.get("id")
        if user_id:
            url = app_settings.DOTMAILER_DELETE_USER_BY_ID_URL.replace("ID", str(user_id))
            delete(url, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))
    except RequestException:
        return HttpResponse(status=404)


def get_dotmailer_user_by_email(email):
    url = app_settings.DOTMAILER_GET_USER_BY_EMAIL_URL.replace("EMAIL", email)

    response = get(url, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))

    return json.loads(response.content)


def add_consent_record_to_dotmailer_user(user):
    consent_date_time = datetime.datetime.now().__str__()

    url = app_settings.DOTMAILER_PUT_CONSENT_DATA_URL.replace("USER_ID", str(user["id"]))
    body = {
        "contact": {
            "email": user["email"],
            "optInType": user["optInType"],
            "emailType": user["emailType"],
            "dataFields": user["dataFields"],
        },
        "consentFields": [{"fields": [{"key": "DATETIMECONSENTED", "value": consent_date_time}]}],
    }

    put(url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))


def send_dotmailer_consent_confirmation_email_to_user(user):
    url = app_settings.DOTMAILER_SEND_CAMPAIGN_URL
    campaign_id = app_settings.DOTMAILER_THANKS_FOR_STAYING_CAMPAIGN_ID
    body = {"campaignID": campaign_id, "contactIds": [str(user["id"])]}

    post(url, json=body, auth=(app_settings.DOTMAILER_USER, app_settings.DOTMAILER_PASSWORD))


def update_indy_email(user, request, data):
    changing_email = False
    new_email = data["email"]

    if new_email != "" and new_email != user.email:
        changing_email = True
        users_with_email = User.objects.filter(email=new_email)
        # email is already taken
        if users_with_email.exists():
            email_message = emailChangeDuplicateNotificationEmail(request, new_email)
            send_email(
                NOTIFICATION_EMAIL,
                [user.email],
                email_message["subject"],
                email_message["message"],
                email_message["subject"],
            )
        else:
            # new email to set and verify
            send_verification_email(request, user, new_email)
    return changing_email, new_email


def update_email(user: Teacher or Student, request, data):
    changing_email = False
    new_email = data["email"]

    if new_email != "" and new_email != user.new_user.email:
        changing_email = True
        users_with_email = User.objects.filter(email=new_email)
        # email is already taken
        if users_with_email.exists():
            email_message = emailChangeDuplicateNotificationEmail(request, new_email)
            send_email(
                NOTIFICATION_EMAIL,
                [user.new_user.email],
                email_message["subject"],
                email_message["message"],
                email_message["subject"],
            )
        else:
            # new email to set and verify
            send_verification_email(request, user.new_user, new_email)
    return changing_email, new_email
